@@ -1,11 +1,12 @@
 {-# LANGUAGE CPP                 #-}
+{-# LANGUAGE LambdaCase          #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TupleSections       #-}
 {-# LANGUAGE OverloadedStrings   #-}
 {-# LANGUAGE FlexibleContexts    #-}
 {- |
    Module      : Text.Pandoc.App.CommandLineOptions
-   Copyright   : Copyright (C) 2006-2020 John MacFarlane
+   Copyright   : Copyright (C) 2006-2022 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley@edu>
@@ -16,6 +17,7 @@ Does a pandoc conversion based on command-line options.
 -}
 module Text.Pandoc.App.CommandLineOptions (
             parseOptions
+          , parseOptionsFromArgs
           , options
           , engines
           , lookupHighlightStyle
@@ -24,16 +26,15 @@ module Text.Pandoc.App.CommandLineOptions (
 import Control.Monad
 import Control.Monad.Trans
 import Control.Monad.Except (throwError)
+import Control.Monad.State.Strict
 import Data.Aeson.Encode.Pretty (encodePretty', Config(..), keyOrder,
          defConfig, Indent(..), NumberFormat(..))
 import Data.Bifunctor (second)
 import Data.Char (toLower)
-import Data.List (intercalate, sort)
+import Data.List (intercalate, sort, foldl')
 #ifdef _WINDOWS
-#if MIN_VERSION_base_noprelude(4,12,0)
 import Data.List (isPrefixOf)
 #endif
-#endif
 import Data.Maybe (fromMaybe, isJust)
 import Data.Text (Text)
 import Safe (tailDef)
@@ -45,10 +46,13 @@ import System.FilePath
 import System.IO (stdout)
 import Text.DocTemplates (Context (..), ToContext (toVal), Val (..))
 import Text.Pandoc
-import Text.Pandoc.App.Opt (Opt (..), LineEnding (..), IpynbOutput (..), addMeta)
+import Text.Pandoc.Builder (setMeta)
+import Text.Pandoc.App.Opt (Opt (..), LineEnding (..), IpynbOutput (..),
+                            DefaultsState (..), applyDefaults,
+                            fullDefaultsPath)
 import Text.Pandoc.Filter (Filter (..))
 import Text.Pandoc.Highlighting (highlightingStyles)
-import Text.Pandoc.Shared (ordNub, elemText, safeStrRead, defaultUserDataDirs, findM)
+import Text.Pandoc.Shared (ordNub, elemText, safeStrRead, defaultUserDataDir)
 import Text.Printf
 
 #ifdef EMBED_DATA_FILES
@@ -63,16 +67,19 @@ import qualified Data.ByteString as BS
 import qualified Data.ByteString.Lazy as B
 import qualified Data.Map as M
 import qualified Data.Text as T
-import qualified Data.YAML as Y
 import qualified Text.Pandoc.UTF8 as UTF8
 
 parseOptions :: [OptDescr (Opt -> IO Opt)] -> Opt -> IO Opt
 parseOptions options' defaults = do
   rawArgs <- map UTF8.decodeArg <$> getArgs
   prg <- getProgName
+  parseOptionsFromArgs options' defaults prg rawArgs
 
+parseOptionsFromArgs
+  :: [OptDescr (Opt -> IO Opt)] -> Opt -> String -> [String] -> IO Opt
+parseOptionsFromArgs options' defaults prg rawArgs = do
   let (actions, args, unrecognizedOpts, errors) =
-           getOpt' Permute options' rawArgs
+           getOpt' Permute options' (map UTF8.decodeArg rawArgs)
 
   let unknownOptionErrors =
        foldr (handleUnrecognizedOption . takeWhile (/= '=')) []
@@ -84,7 +91,7 @@ parseOptions options' defaults = do
         ("Try " ++ prg ++ " --help for more information.")
 
   -- thread option data structure through all supplied option actions
-  opts <- foldl (>>=) (return defaults) actions
+  opts <- foldl' (>>=) (return defaults) actions
   let mbArgs = case args of
                  [] -> Nothing
                  xs -> Just xs
@@ -102,7 +109,7 @@ latexEngines :: [String]
 latexEngines  = ["pdflatex", "lualatex", "xelatex", "latexmk", "tectonic"]
 
 htmlEngines :: [String]
-htmlEngines  = ["wkhtmltopdf", "weasyprint", "prince"]
+htmlEngines  = ["pagedjs-cli", "wkhtmltopdf", "weasyprint", "prince"]
 
 engines :: [(Text, String)]
 engines = map ("html",) htmlEngines ++
@@ -165,7 +172,11 @@ options =
 
     , Option "d" ["defaults"]
                  (ReqArg
-                  (\arg opt -> applyDefaults opt arg
+                  (\arg opt -> runIOorExplode $ do
+                     let defsState = DefaultsState { curDefaults = Nothing,
+                                                     inheritanceGraph = [] }
+                     fp <- fullDefaultsPath (optDataDir opt) arg
+                     evalStateT (applyDefaults opt fp) defsState
                   )
                   "FILE")
                 ""
@@ -175,6 +186,11 @@ options =
                   (\opt -> return opt { optFileScope = True }))
                  "" -- "Parse input files before combining"
 
+    , Option "" ["sandbox"]
+                 (NoArg
+                  (\opt -> return opt { optSandbox = True }))
+                 ""
+
     , Option "s" ["standalone"]
                  (NoArg
                   (\opt -> return opt { optStandalone = True }))
@@ -275,28 +291,32 @@ options =
     , Option "" ["resource-path"]
                 (ReqArg
                   (\arg opt -> return opt { optResourcePath =
-                                   splitSearchPath arg })
+                                   splitSearchPath arg ++
+                                    optResourcePath opt })
                    "SEARCHPATH")
                   "" -- "Paths to search for images and other resources"
 
     , Option "H" ["include-in-header"]
                  (ReqArg
                   (\arg opt -> return opt{ optIncludeInHeader =
-                                             optIncludeInHeader opt ++ [arg] })
+                                             optIncludeInHeader opt ++
+                                             [normalizePath arg] })
                   "FILE")
                  "" -- "File to include at end of header (implies -s)"
 
     , Option "B" ["include-before-body"]
                  (ReqArg
                   (\arg opt -> return opt{ optIncludeBeforeBody =
-                                            optIncludeBeforeBody opt ++ [arg] })
+                                            optIncludeBeforeBody opt ++
+                                            [normalizePath arg] })
                   "FILE")
                  "" -- "File to include before document body"
 
     , Option "A" ["include-after-body"]
                  (ReqArg
                   (\arg opt -> return opt{ optIncludeAfterBody =
-                                            optIncludeAfterBody opt ++ [arg] })
+                                            optIncludeAfterBody opt ++
+                                            [normalizePath arg] })
                   "FILE")
                  "" -- "File to include after document body"
 
@@ -308,20 +328,15 @@ options =
     , Option "" ["highlight-style"]
                 (ReqArg
                  (\arg opt ->
-                     return opt{ optHighlightStyle = Just $ T.pack arg })
+                     return opt{ optHighlightStyle = Just $
+                                 T.pack $ normalizePath arg })
                  "STYLE|FILE")
                  "" -- "Style for highlighted code"
 
     , Option "" ["syntax-definition"]
                 (ReqArg
-                 (\arg opt -> do
-                   let tr c d = map (\x -> if x == c then d else x)
-                   let arg' = case arg of -- see #4836
-                                   -- HXT confuses Windows path with URI
-                                   _:':':'\\':_ ->
-                                       "file:///" ++ tr '\\' '/' arg
-                                   _ -> normalizePath arg
-                   return opt{ optSyntaxDefinitions = arg' :
+                 (\arg opt ->
+                   return opt{ optSyntaxDefinitions = normalizePath arg :
                                 optSyntaxDefinitions opt })
                  "FILE")
                 "" -- "Syntax definition (xml) file"
@@ -396,7 +411,7 @@ options =
     , Option "" ["reference-doc"]
                  (ReqArg
                   (\arg opt ->
-                    return opt { optReferenceDoc = Just arg })
+                    return opt { optReferenceDoc = Just $ normalizePath arg })
                   "FILE")
                  "" -- "Path of custom reference doc"
 
@@ -414,9 +429,15 @@ options =
                   "NAME:VALUE")
                  ""
 
+    , Option "" ["no-check-certificate"]
+                (NoArg
+                 (\opt -> return opt { optNoCheckCertificate = True }))
+                "" -- "Disable certificate validation"
+
     , Option "" ["abbreviations"]
                 (ReqArg
-                 (\arg opt -> return opt { optAbbreviations = Just arg })
+                 (\arg opt -> return opt { optAbbreviations =
+                                            Just $ normalizePath arg })
                 "FILE")
                 "" -- "Specify file for custom abbreviations"
 
@@ -518,9 +539,26 @@ options =
 
     , Option "" ["atx-headers"]
                  (NoArg
-                  (\opt -> return opt { optSetextHeaders = False } ))
+                  (\opt -> do
+                    deprecatedOption "--atx-headers"
+                      "Use --markdown-headings=atx instead."
+                    return opt { optSetextHeaders = False } ))
                  "" -- "Use atx-style headers for markdown"
 
+    , Option "" ["markdown-headings"]
+                  (ReqArg
+                    (\arg opt -> do
+                      headingFormat <- case arg of
+                        "setext" -> pure True
+                        "atx" -> pure False
+                        _ -> E.throwIO $ PandocOptionError $ T.pack
+                          ("Unknown markdown heading format: " ++ arg ++
+                            ". Expecting atx or setext")
+                      pure opt { optSetextHeaders = headingFormat }
+                    )
+                  "setext|atx")
+                  ""
+
     , Option "" ["listings"]
                  (NoArg
                   (\opt -> return opt { optListings = True }))
@@ -535,10 +573,10 @@ options =
                  (ReqArg
                   (\arg opt ->
                       case safeStrRead arg of
-                           Just t | t >= 1 && t <= 6 ->
+                           Just t | t >= 0 && t <= 6 ->
                                     return opt { optSlideLevel = Just t }
                            _      -> E.throwIO $ PandocOptionError
-                                    "slide level must be a number between 1 and 6")
+                                    "slide level must be a number between 0 and 6")
                  "NUMBER")
                  "" -- "Force header level for slides"
 
@@ -601,21 +639,24 @@ options =
                  (ReqArg
                   (\arg opt ->
                      return opt { optVariables =
-                       setVariable "epub-cover-image" (T.pack arg) $
+                       setVariable "epub-cover-image"
+                         (T.pack $ normalizePath arg) $
                          optVariables opt })
                   "FILE")
                  "" -- "Path of epub cover image"
 
     , Option "" ["epub-metadata"]
                  (ReqArg
-                  (\arg opt -> return opt { optEpubMetadata = Just arg })
+                  (\arg opt -> return opt { optEpubMetadata = Just $
+                                             normalizePath arg })
                   "FILE")
                  "" -- "Path of epub metadata file"
 
     , Option "" ["epub-embed-font"]
                  (ReqArg
                   (\arg opt ->
-                     return opt{ optEpubFonts = arg : optEpubFonts opt })
+                     return opt{ optEpubFonts = normalizePath arg :
+                                                optEpubFonts opt })
                   "FILE")
                  "" -- "Directory of fonts to embed"
 
@@ -642,10 +683,17 @@ options =
                  "all|none|best")
                  "" -- "Starting number for sections, subsections, etc."
 
+    , Option "C" ["citeproc"]
+                 (NoArg
+                  (\opt -> return opt { optFilters =
+                      optFilters opt ++ [CiteprocFilter] }))
+                 "" -- "Process citations"
+
     , Option "" ["bibliography"]
                  (ReqArg
                   (\arg opt -> return opt{ optMetadata =
-                                            addMeta "bibliography" arg $
+                                            addMeta "bibliography"
+                                              (normalizePath arg) $
                                               optMetadata opt })
                    "FILE")
                  ""
@@ -654,7 +702,8 @@ options =
                  (ReqArg
                   (\arg opt ->
                      return opt{ optMetadata =
-                                   addMeta "csl" arg $ optMetadata opt })
+                                   addMeta "csl" (normalizePath arg) $
+                                   optMetadata opt })
                    "FILE")
                  ""
 
@@ -662,8 +711,8 @@ options =
                  (ReqArg
                   (\arg opt ->
                      return opt{ optMetadata =
-                                  addMeta "citation-abbreviations" arg $
-                                    optMetadata opt })
+                                  addMeta "citation-abbreviations"
+                                    (normalizePath arg) $ optMetadata opt })
                    "FILE")
                  ""
 
@@ -694,8 +743,7 @@ options =
     , Option "" ["mathjax"]
                  (OptArg
                   (\arg opt -> do
-                      let url' = maybe (defaultMathJaxURL <>
-                                  "tex-mml-chtml.js") T.pack arg
+                      let url' = maybe defaultMathJaxURL T.pack arg
                       return opt { optHTMLMathMethod = MathJax url'})
                   "URL")
                  "" -- "Use MathJax for HTML math"
@@ -747,7 +795,8 @@ options =
 
     , Option "" ["log"]
                  (ReqArg
-                  (\arg opt -> return opt{ optLogFile = Just arg })
+                  (\arg opt -> return opt{ optLogFile = Just $
+                                            normalizePath arg })
                 "FILE")
                 "" -- "Log messages in JSON format to this file."
 
@@ -762,10 +811,10 @@ options =
                            map (\c -> ['-',c]) shorts ++
                            map ("--" ++) longs
                      let allopts = unwords (concatMap optnames options)
-                     UTF8.hPutStrLn stdout $ printf tpl allopts
-                         (unwords readersNames)
-                         (unwords writersNames)
-                         (unwords $ map (T.unpack . fst) highlightingStyles)
+                     UTF8.hPutStrLn stdout $ T.pack $ printf tpl allopts
+                         (T.unpack $ T.unwords readersNames)
+                         (T.unpack $ T.unwords writersNames)
+                         (T.unpack $ T.unwords $ map fst highlightingStyles)
                          (unwords datafiles)
                      exitSuccess ))
                  "" -- "Print bash completion script"
@@ -793,20 +842,23 @@ options =
                            case arg of
                              Nothing  -> extensionsFromList extList
                              Just fmt -> getAllExtensions $ T.pack fmt
-                     let defExts =
-                           case arg of
-                             Nothing   -> getDefaultExtensions
-                                           "markdown"
-                             Just fmt  -> getDefaultExtensions $ T.pack fmt
-                     let showExt x =
-                           (if extensionEnabled x defExts
-                               then '+'
-                               else if extensionEnabled x allExts
-                                       then '-'
-                                       else ' ') : drop 4 (show x)
-                     mapM_ (UTF8.hPutStrLn stdout . showExt)
-                       [ex | ex <- extList, extensionEnabled ex allExts]
-                     exitSuccess )
+                     let formatName = maybe "markdown" T.pack arg
+                     if formatName `notElem`
+                         (map fst (readers :: [(Text, Reader PandocPure)]) ++
+                          map fst (writers :: [(Text, Writer PandocPure)]))
+                        then E.throwIO $ PandocOptionError $ formatName <>
+                               " is not a recognized reader or writer format"
+                        else do
+                          let defExts = getDefaultExtensions formatName
+                          let showExt x =
+                               (if extensionEnabled x defExts
+                                   then '+'
+                                   else if extensionEnabled x allExts
+                                           then '-'
+                                           else ' ') : drop 4 (show x)
+                          mapM_ (UTF8.hPutStrLn stdout . T.pack . showExt)
+                             [ex | ex <- extList, extensionEnabled ex allExts]
+                          exitSuccess )
                   "FORMAT")
                  ""
 
@@ -818,14 +870,14 @@ options =
                                  , sShortname s `notElem`
                                     [T.pack "Alert", T.pack "Alert_indent"]
                                  ]
-                     mapM_ (UTF8.hPutStrLn stdout) langs
+                     mapM_ (UTF8.hPutStrLn stdout . T.pack) (sort langs)
                      exitSuccess ))
                  ""
 
     , Option "" ["list-highlight-styles"]
                  (NoArg
                   (\_ -> do
-                     mapM_ (UTF8.hPutStrLn stdout . T.unpack . fst) highlightingStyles
+                     mapM_ (UTF8.hPutStrLn stdout . fst) highlightingStyles
                      exitSuccess ))
                  ""
 
@@ -843,7 +895,7 @@ options =
                             | T.null t -> -- e.g. for docx, odt, json:
                                 E.throwIO $ PandocCouldNotFindDataFileError $ T.pack
                                   ("templates/default." ++ arg)
-                            | otherwise -> write . T.unpack $ t
+                            | otherwise -> write t
                           Left e  -> E.throwIO e
                      exitSuccess)
                   "FORMAT")
@@ -864,9 +916,7 @@ options =
     , Option "" ["print-highlight-style"]
                  (ReqArg
                   (\arg opt -> do
-                     let write = case optOutputFile opt of
-                                        Just f  -> B.writeFile f
-                                        Nothing -> B.putStr
+                     let write = maybe B.putStr B.writeFile $ optOutputFile opt
                      sty <- runIOorExplode $ lookupHighlightStyle arg
                      write $ encodePretty'
                        defConfig{confIndent = Spaces 4
@@ -891,11 +941,13 @@ options =
                  (NoArg
                   (\_ -> do
                      prg <- getProgName
-                     defaultDatadirs <- defaultUserDataDirs
-                     UTF8.hPutStrLn stdout (prg ++ " " ++ T.unpack pandocVersion ++
-                       compileInfo ++ "\nDefault user data directory: " ++
-                       intercalate " or " defaultDatadirs ++
-                       ('\n':copyrightMessage))
+                     defaultDatadir <- defaultUserDataDir
+                     UTF8.hPutStrLn stdout
+                      $ T.pack
+                      $ prg ++ " " ++ T.unpack pandocVersion ++
+                        compileInfo ++
+                        "\nUser data directory: " ++ defaultDatadir ++
+                        ('\n':copyrightMessage)
                      exitSuccess ))
                  "" -- "Print version"
 
@@ -903,7 +955,7 @@ options =
                  (NoArg
                   (\_ -> do
                      prg <- getProgName
-                     UTF8.hPutStr stdout (usageMessage prg options)
+                     UTF8.hPutStr stdout (T.pack $ usageMessage prg options)
                      exitSuccess ))
                  "" -- "Show help"
     ]
@@ -924,16 +976,16 @@ usageMessage programName = usageInfo (programName ++ " [OPTIONS] [FILES]")
 
 copyrightMessage :: String
 copyrightMessage = intercalate "\n" [
-  "Copyright (C) 2006-2020 John MacFarlane",
-  "Web:  https://pandoc.org",
-  "This is free software; see the source for copying conditions.",
-  "There is no warranty, not even for merchantability or fitness",
-  "for a particular purpose." ]
+ "Copyright (C) 2006-2022 John MacFarlane. Web:  https://pandoc.org",
+ "This is free software; see the source for copying conditions. There is no",
+ "warranty, not even for merchantability or fitness for a particular purpose." ]
 
 compileInfo :: String
 compileInfo =
-  "\nCompiled with pandoc-types " ++ VERSION_pandoc_types ++ ", texmath " ++
-  VERSION_texmath ++ ", skylighting " ++ VERSION_skylighting
+  "\nCompiled with pandoc-types " ++ VERSION_pandoc_types ++
+  ", texmath " ++ VERSION_texmath ++ ", skylighting " ++
+  VERSION_skylighting ++ ",\nciteproc " ++ VERSION_citeproc ++
+  ", ipynb " ++ VERSION_ipynb
 
 handleUnrecognizedOption :: String -> [String] -> [String]
 handleUnrecognizedOption "--smart" =
@@ -964,38 +1016,16 @@ handleUnrecognizedOption "-R" = handleUnrecognizedOption "--parse-raw"
 handleUnrecognizedOption x =
   (("Unknown option " ++ x ++ ".") :)
 
-readersNames :: [String]
-readersNames = sort (map (T.unpack . fst) (readers :: [(Text, Reader PandocIO)]))
+readersNames :: [Text]
+readersNames = sort (map fst (readers :: [(Text, Reader PandocIO)]))
 
-writersNames :: [String]
+writersNames :: [Text]
 writersNames = sort
-  ("pdf" : map (T.unpack . fst) (writers :: [(Text, Writer PandocIO)]))
+  ("pdf" : map fst (writers :: [(Text, Writer PandocIO)]))
 
 splitField :: String -> (String, String)
 splitField = second (tailDef "true") . break (`elemText` ":=")
 
--- | Apply defaults from --defaults file.
-applyDefaults :: Opt -> FilePath -> IO Opt
-applyDefaults opt file = runIOorExplode $ do
-  let fp = if null (takeExtension file)
-              then addExtension file "yaml"
-              else file
-  setVerbosity $ optVerbosity opt
-  dataDirs <- liftIO defaultUserDataDirs
-  let fps = fp : case optDataDir opt of
-              Nothing -> map (</> ("defaults" </> fp))
-                               dataDirs
-              Just dd -> [dd </> "defaults" </> fp]
-  fp' <- fromMaybe fp <$> findM fileExists fps
-  inp <- readFileLazy fp'
-  case Y.decode1 inp of
-      Right (f :: Opt -> Opt) -> return $ f opt
-      Left (errpos, errmsg)  -> throwError $
-         PandocParseError $ T.pack $
-         "Error parsing " ++ fp' ++ " line " ++
-          show (Y.posLine errpos) ++ " column " ++
-          show (Y.posColumn errpos) ++ ":\n" ++ errmsg
-
 lookupHighlightStyle :: PandocMonad m => String -> m Style
 lookupHighlightStyle s
   | takeExtension s == ".theme" = -- attempt to load KDE theme
@@ -1013,7 +1043,7 @@ lookupHighlightStyle s
 deprecatedOption :: String -> String -> IO ()
 deprecatedOption o msg =
   runIO (report $ Deprecated (T.pack o) (T.pack msg)) >>=
-    \r -> case r of
+    \case
        Right () -> return ()
        Left e   -> E.throwIO e
 
@@ -1024,11 +1054,31 @@ setVariable key val (Context ctx) = Context $ M.alter go key ctx
         go (Just (ListVal xs)) = Just $ ListVal $ xs ++ [toVal val]
         go (Just x)            = Just $ ListVal [x, toVal val]
 
+addMeta :: String -> String -> Meta -> Meta
+addMeta k v meta =
+  case lookupMeta k' meta of
+       Nothing -> setMeta k' v' meta
+       Just (MetaList xs) ->
+                  setMeta k' (MetaList (xs ++ [v'])) meta
+       Just x  -> setMeta k' (MetaList [x, v']) meta
+ where
+  v' = readMetaValue v
+  k' = T.pack k
+
+readMetaValue :: String -> MetaValue
+readMetaValue s
+  | s == "true"  = MetaBool True
+  | s == "True"  = MetaBool True
+  | s == "TRUE"  = MetaBool True
+  | s == "false" = MetaBool False
+  | s == "False" = MetaBool False
+  | s == "FALSE" = MetaBool False
+  | otherwise    = MetaString $ T.pack s
+
 -- On Windows with ghc 8.6+, we need to rewrite paths
 -- beginning with \\ to \\?\UNC\. -- See #5127.
 normalizePath :: FilePath -> FilePath
 #ifdef _WINDOWS
-#if MIN_VERSION_base_noprelude(4,12,0)
 normalizePath fp =
   if "\\\\" `isPrefixOf` fp && not ("\\\\?\\" `isPrefixOf` fp)
     then "\\\\?\\UNC\\" ++ drop 2 fp
@@ -1036,6 +1086,3 @@ normalizePath fp =
 #else
 normalizePath = id
 #endif
-#else
-normalizePath = id
-#endif
