@@ -1,7 +1,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 {- |
    Module      : Text.Pandoc.App.FormatHeuristics
-   Copyright   : Copyright (C) 2006-2020 John MacFarlane
+   Copyright   : Copyright (C) 2006-2022 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley@edu>
@@ -15,18 +15,24 @@ module Text.Pandoc.App.FormatHeuristics
   ) where
 
 import Data.Char (toLower)
+import Data.Foldable (asum)
 import Data.Text (Text)
 import System.FilePath (takeExtension)
 
--- Determine default format based on file extensions.
+-- | Determines default format based on file extensions; uses the format
+-- of the first extension that's associated with a format.
+--
+-- Examples:
+--
+-- > formatFromFilePaths ["text.unknown", "no-extension"]
+-- Nothing
+--
+-- > formatFromFilePaths ["my.md", "other.rst"]
+-- Just "markdown"
 formatFromFilePaths :: [FilePath] -> Maybe Text
-formatFromFilePaths [] = Nothing
-formatFromFilePaths (x:xs) =
-  case formatFromFilePath x of
-    Just f     -> Just f
-    Nothing    -> formatFromFilePaths xs
+formatFromFilePaths = asum . map formatFromFilePath
 
--- Determine format based on file extension
+-- | Determines format based on file extension.
 formatFromFilePath :: FilePath -> Maybe Text
 formatFromFilePath x =
   case takeExtension (map toLower x) of
@@ -48,6 +54,12 @@ formatFromFilePath x =
     ".lhs"      -> Just "markdown+lhs"
     ".ltx"      -> Just "latex"
     ".markdown" -> Just "markdown"
+    ".markua"   -> Just "markua"
+    ".mkdn"     -> Just "markdown"
+    ".mkd"      -> Just "markdown"
+    ".mdwn"     -> Just "markdown"
+    ".mdown"    -> Just "markdown"
+    ".Rmd"      -> Just "markdown"
     ".md"       -> Just "markdown"
     ".ms"       -> Just "ms"
     ".muse"     -> Just "muse"
@@ -63,7 +75,6 @@ formatFromFilePath x =
     ".s5"       -> Just "s5"
     ".t2t"      -> Just "t2t"
     ".tei"      -> Just "tei"
-    ".tei.xml"  -> Just "tei"
     ".tex"      -> Just "latex"
     ".texi"     -> Just "texinfo"
     ".texinfo"  -> Just "texinfo"
@@ -74,5 +85,6 @@ formatFromFilePath x =
     ".xhtml"    -> Just "html"
     ".ipynb"    -> Just "ipynb"
     ".csv"      -> Just "csv"
+    ".bib"      -> Just "biblatex"
     ['.',y]     | y `elem` ['1'..'9'] -> Just "man"
     _           -> Nothing
