@@ -21,8 +21,13 @@ import Text.Parsec
 import Text.Parsec.Text (Parser)
 
 type URL = T.Text
+type Anchor = T.Text
 
 data FieldInfo = HyperlinkField URL
+                -- The boolean indicates whether the field is a hyperlink.
+               | PagerefField Anchor Bool
+               | ZoteroItem T.Text
+               | ZoteroBibliography
                | UnknownField
                deriving (Show)
 
@@ -33,8 +38,32 @@ fieldInfo :: Parser FieldInfo
 fieldInfo =
   try (HyperlinkField <$> hyperlink)
   <|>
+  try ((uncurry PagerefField) <$> pageref) 
+  <|>
+  try addIn
+  <|>
   return UnknownField
 
+addIn :: Parser FieldInfo
+addIn = do
+  spaces
+  string "ADDIN"
+  spaces
+  try zoteroItem <|> zoteroBibliography
+
+zoteroItem :: Parser FieldInfo
+zoteroItem = do
+  string "ZOTERO_ITEM"
+  spaces
+  string "CSL_CITATION"
+  spaces
+  ZoteroItem <$> getInput
+
+zoteroBibliography :: Parser FieldInfo
+zoteroBibliography = do
+  string "ZOTERO_BIBL"
+  return ZoteroBibliography
+
 escapedQuote :: Parser T.Text
 escapedQuote = string "\\\"" $> "\\\""
 
@@ -72,3 +101,23 @@ hyperlink = do
               ("\\l", s) : _ -> farg <> "#" <> s
               _              -> farg
   return url
+
+-- See §17.16.5.45
+pagerefSwitch :: Parser (T.Text, T.Text)
+pagerefSwitch = do
+  sw <- string "\\h"
+  spaces
+  farg <- fieldArgument
+  return (T.pack sw, farg)
+
+pageref :: Parser (Anchor, Bool)
+pageref = do
+  many space
+  string "PAGEREF"
+  spaces
+  farg <- fieldArgument
+  switches <- spaces *> many pagerefSwitch
+  let isLink = case switches of
+              ("\\h", _) : _ -> True 
+              _              -> False
+  return (farg, isLink)
