@@ -20,7 +20,7 @@ import Control.Monad (liftM, mzero, guard, void)
 import Control.Monad.Trans (lift)
 import Control.Monad.Except (throwError)
 import Data.Maybe (catMaybes, isJust)
-import Data.List (intersperse, intercalate)
+import Data.List (intersperse)
 import qualified Data.Text as T
 import Text.Pandoc.Builder as B
 import Text.Pandoc.Class.PandocMonad (PandocMonad(..), report)
@@ -29,11 +29,10 @@ import Text.Pandoc.Logging (LogMessage(..))
 import Text.Pandoc.Options
 import Text.Pandoc.Parsing
 import Text.Pandoc.Walk (query)
-import Text.Pandoc.Shared (crFilter, mapLeft)
+import Text.Pandoc.Shared (mapLeft)
 import Text.Pandoc.Readers.Roff  -- TODO explicit imports
-import Text.Parsec hiding (tokenPrim)
 import qualified Text.Parsec as Parsec
-import Text.Parsec.Pos (updatePosString, initialPos)
+import Text.Parsec.Pos (updatePosString)
 import qualified Data.Foldable as Foldable
 
 data ManState = ManState { readerOptions   :: ReaderOptions
@@ -50,13 +49,20 @@ type ManParser m = ParserT [RoffToken] ManState m
 
 
 -- | Read man (troff) from an input string and return a Pandoc document.
-readMan :: PandocMonad m => ReaderOptions -> T.Text -> m Pandoc
-readMan opts txt = do
-  tokenz <- lexRoff (initialPos "input") (crFilter txt)
+readMan :: (PandocMonad m, ToSources a)
+        => ReaderOptions
+        -> a
+        -> m Pandoc
+readMan opts s = do
+  let Sources inps = toSources s
+  tokenz <- mconcat <$> mapM (uncurry lexRoff) inps
   let state = def {readerOptions = opts} :: ManState
+  let fixError (PandocParsecError _ e) = PandocParsecError (Sources inps) e
+      fixError e = e
   eitherdoc <- readWithMTokens parseMan state
      (Foldable.toList . unRoffTokens $ tokenz)
-  either throwError return eitherdoc
+  either (throwError . fixError) return eitherdoc
+
 
 readWithMTokens :: PandocMonad m
         => ParserT [RoffToken] ManState m a  -- ^ parser
@@ -64,9 +70,10 @@ readWithMTokens :: PandocMonad m
         -> [RoffToken]                       -- ^ input
         -> m (Either PandocError a)
 readWithMTokens parser state input =
-  let leftF = PandocParsecError . T.pack . intercalate "\n" $ show <$> input
+  let leftF = PandocParsecError mempty
   in mapLeft leftF `liftM` runParserT parser state "source" input
 
+
 parseMan :: PandocMonad m => ManParser m Pandoc
 parseMan = do
   bs <- many parseBlock <* eof
@@ -89,7 +96,7 @@ parseBlock = choice [ parseList
 
 parseTable :: PandocMonad m => ManParser m Blocks
 parseTable = do
-  modifyState $ \st -> st { tableCellsPlain = True }
+  updateState $ \st -> st { tableCellsPlain = True }
   let isTbl Tbl{} = True
       isTbl _     = False
   Tbl _opts rows pos <- msatisfy isTbl
@@ -107,11 +114,12 @@ parseTable = do
       bodyRows <- mapM (mapM parseTableCell . snd) bodyRows'
       isPlainTable <- tableCellsPlain <$> getState
       let widths = if isPlainTable
-                      then repeat 0.0
-                      else repeat ((1.0 / fromIntegral (length alignments))
-                                   :: Double)
-      return $ B.table mempty (zip alignments widths)
-                  headerRow bodyRows) <|> fallback pos
+                      then repeat ColWidthDefault
+                      else repeat $ ColWidth (1.0 / fromIntegral (length alignments))
+      return $ B.table B.emptyCaption (zip alignments widths)
+                  (TableHead nullAttr $ toHeaderRow headerRow)
+                  [TableBody nullAttr 0 [] $ map toRow bodyRows]
+                  (TableFoot nullAttr [])) <|> fallback pos
     [] -> fallback pos
 
  where
@@ -134,7 +142,7 @@ parseTable = do
         case res' of
           Left _  -> Prelude.fail "Could not parse table cell"
           Right x -> do
-            modifyState $ \s -> s{ tableCellsPlain = False }
+            updateState $ \s -> s{ tableCellsPlain = False }
             return x
       Right x -> return x
 
@@ -160,6 +168,8 @@ parseTable = do
       'r' -> Just AlignRight
       _   -> Nothing
 
+  toRow = Row nullAttr . map simpleCell
+  toHeaderRow l = [toRow l | not (null l)]
 
 parseNewParagraph :: PandocMonad m => ManParser m Blocks
 parseNewParagraph = do
@@ -219,7 +229,7 @@ parseTitle = do
                       setMeta "section" (linePartsToInlines y)
          [x]       -> setMeta "title" (linePartsToInlines x)
          []        -> id
-  modifyState $ \st -> st{ metadata = adjustMeta $ metadata st }
+  updateState $ \st -> st{ metadata = adjustMeta $ metadata st }
   return mempty
 
 linePartsToInlines :: [LinePart] -> Inlines
@@ -404,12 +414,14 @@ parseBlockQuote = blockQuote <$> continuation
 
 data ListType = Ordered ListAttributes
               | Bullet
+              | Definition T.Text
 
 listTypeMatches :: Maybe ListType -> ListType -> Bool
 listTypeMatches Nothing _            = True
 listTypeMatches (Just Bullet) Bullet = True
 listTypeMatches (Just (Ordered (_,x,y))) (Ordered (_,x',y'))
                                      = x == x' && y == y'
+listTypeMatches (Just (Definition _)) (Definition _) = True
 listTypeMatches (Just _) _           = False
 
 listItem :: PandocMonad m => Maybe ListType -> ManParser m (ListType, Blocks)
@@ -424,20 +436,28 @@ listItem mbListType = try $ do
                   Right (start, listtype, listdelim)
                     | cs == cs' -> Ordered (start, listtype, listdelim)
                     | otherwise -> Ordered (start, listtype, DefaultDelim)
-                  Left _        -> Bullet
+                  Left _
+                    | cs == "\183" || cs == "-" || cs == "*" || cs == "+"
+                                   -> Bullet
+                    | otherwise    -> Definition cs
       guard $ listTypeMatches mbListType lt
+      skipMany memptyLine
       inls <- option mempty parseInlines
+      skipMany memptyLine
       continuations <- mconcat <$> many continuation
       return (lt, para inls <> continuations)
     []          -> mzero
 
 parseList :: PandocMonad m => ManParser m Blocks
 parseList = try $ do
-  (lt, x) <- listItem Nothing
-  xs <- map snd <$> many (listItem (Just lt))
+  x@(lt, _) <- listItem Nothing
+  xs <- many (listItem (Just lt))
+  let toDefItem (Definition t, bs) = (B.text t, [bs])
+      toDefItem _ = mempty
   return $ case lt of
-             Bullet        -> bulletList (x:xs)
-             Ordered lattr -> orderedListWith lattr (x:xs)
+             Bullet        -> bulletList $ map snd (x:xs)
+             Ordered lattr -> orderedListWith lattr $ map snd (x:xs)
+             Definition _  -> definitionList $ map toDefItem (x:xs)
 
 continuation :: PandocMonad m => ManParser m Blocks
 continuation =
@@ -450,11 +470,15 @@ definitionListItem :: PandocMonad m
                    => ManParser m (Inlines, [Blocks])
 definitionListItem = try $ do
   mmacro "TP"  -- args specify indent level, can ignore
+  skipMany memptyLine
   term <- parseInline
+  skipMany memptyLine
   moreterms <- many $ try $ do
                  mmacro "TQ"
                  parseInline
+  skipMany memptyLine
   inls <- option mempty parseInlines
+  skipMany memptyLine
   continuations <- mconcat <$> many continuation
   return ( mconcat (intersperse B.linebreak (term:moreterms))
          , [para inls <> continuations])
