@@ -1,10 +1,9 @@
 {-# LANGUAGE CPP #-}
-{-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE TupleSections #-}
 {-# LANGUAGE FlexibleInstances    #-}
-{-# LANGUAGE TypeSynonymInstances #-}
 {- |
    Module      : Tests.Helpers
-   Copyright   : © 2006-2020 John MacFarlane
+   Copyright   : © 2006-2022 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley@edu>
@@ -15,8 +14,9 @@ Utility functions for the test suite.
 -}
 module Tests.Helpers ( test
                      , TestResult(..)
+                     , setupEnvironment
                      , showDiff
-                     , findPandoc
+                     , testGolden
                      , (=?>)
                      , purely
                      , ToString(..)
@@ -24,15 +24,16 @@ module Tests.Helpers ( test
                      )
                      where
 
-import Prelude
+import System.FilePath
 import Data.Algorithm.Diff
 import qualified Data.Map as M
+import qualified Text.Pandoc.UTF8 as UTF8
 import Data.Text (Text, unpack)
-import System.Directory
-import System.Environment.Executable (getExecutablePath)
+import qualified Data.Text as T
 import System.Exit
-import System.FilePath
+import qualified System.Environment as Env
 import Test.Tasty
+import Test.Tasty.Golden.Advanced (goldenTest)
 import Test.Tasty.HUnit
 import Text.Pandoc.Builder (Blocks, Inlines, doc, plain)
 import Text.Pandoc.Class
@@ -42,7 +43,7 @@ import Text.Pandoc.Shared (trimr)
 import Text.Pandoc.Writers.Native (writeNative)
 import Text.Printf
 
-test :: (ToString a, ToString b, ToString c)
+test :: (ToString a, ToString b, ToString c, HasCallStack)
      => (a -> b)  -- ^ function to test
      -> String    -- ^ name of test case
      -> (a, c)    -- ^ (input, expected value)
@@ -64,6 +65,42 @@ test fn name (input, expected) =
            dashes "" = replicate 72 '-'
            dashes x  = replicate (72 - length x - 5) '-' ++ " " ++ x ++ " ---"
 
+testGolden :: TestName -> FilePath -> FilePath -> (Text -> IO Text) -> TestTree
+testGolden  name expectedPath inputPath fn =
+  goldenTest
+    name
+    (UTF8.readFile expectedPath)
+    (UTF8.readFile inputPath >>= fn)
+    compareVals
+    (UTF8.writeFile expectedPath)
+ where
+  compareVals expected actual
+    | expected == actual = return Nothing
+    | otherwise =  return $ Just $
+        "\n--- " ++ expectedPath ++ "\n+++\n" ++
+        showDiff (1,1)
+          (getDiff (lines . filter (/='\r') $ T.unpack actual)
+                   (lines . filter (/='\r') $ T.unpack expected))
+
+-- | Set up environment for pandoc command tests.
+setupEnvironment :: FilePath -> IO [(String, String)]
+setupEnvironment testExePath = do
+  mldpath   <- Env.lookupEnv "LD_LIBRARY_PATH"
+  mdyldpath <- Env.lookupEnv "DYLD_LIBRARY_PATH"
+  mpdd <- Env.lookupEnv "pandoc_datadir"
+  -- Note that Cabal sets the pandoc_datadir environment variable
+  -- to point to the source directory, since otherwise getDataFilename
+  -- will look in the data directory into which pandoc will be installed
+  -- (but has not yet been).  So when we spawn a new process with
+  -- pandoc, we need to make sure this environment variable is set.
+  return $ ("PATH",takeDirectory testExePath) :
+           ("TMP",".") :
+           ("LANG","en_US.UTF-8") :
+           ("HOME", "./") :
+           maybe [] ((:[]) . ("pandoc_datadir",)) mpdd ++
+           maybe [] ((:[]) . ("LD_LIBRARY_PATH",)) mldpath ++
+           maybe [] ((:[]) . ("DYLD_LIBRARY_PATH",)) mdyldpath
+
 data TestResult = TestPassed
                 | TestError ExitCode
                 | TestFailed String FilePath [Diff String]
@@ -87,34 +124,6 @@ showDiff (l,r) (Second ln : ds) =
 showDiff (l,r) (Both _ _ : ds) =
   showDiff (l+1,r+1) ds
 
--- | Find pandoc executable relative to test-pandoc
-findPandoc :: IO FilePath
-findPandoc = do
-  testExePath <- getExecutablePath
-  let pandocDir =
-        case reverse (splitDirectories (takeDirectory testExePath)) of
-             -- cabalv2 with --disable-optimization
-             "test-pandoc" : "build" : "noopt" : "test-pandoc" : "t" : ps
-               -> joinPath (reverse ps) </>
-                  "x" </> "pandoc" </> "noopt" </> "build" </> "pandoc"
-             -- cabalv2 without --disable-optimization
-             "test-pandoc" : "build" : "test-pandoc" : "t" : ps
-               -> joinPath (reverse ps) </>
-                  "x" </> "pandoc" </> "build" </> "pandoc"
-             -- cabalv1
-             "test-pandoc" : "build" : ps
-               -> joinPath (reverse ps) </> "build" </> "pandoc"
-             _ -> error $ "findPandoc: could not find pandoc executable"
-  let pandocPath = pandocDir </> "pandoc"
-#ifdef _WINDOWS
-                             <.> "exe"
-#endif
-  found <- doesFileExist pandocPath
-  if found
-     then return pandocPath
-     else error $ "findPandoc: could not find pandoc executable at "
-                   ++ pandocPath
-
 vividize :: Diff String -> String
 vividize (Both s _) = "  " ++ s
 vividize (First s)  = "- " ++ s
