@@ -1,4 +1,3 @@
-{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE OverloadedStrings #-}
 module Tests.Writers.Muse (tests) where
 
@@ -372,8 +371,12 @@ tests = [ testGroup "block elements"
             [ "table without header" =:
               let rows = [[para "Para 1.1", para "Para 1.2"]
                          ,[para "Para 2.1", para "Para 2.2"]]
-              in table mempty [(AlignDefault,0.0),(AlignDefault,0.0)]
-                       [mempty, mempty] rows
+                  toRow = Row nullAttr . map simpleCell
+              in table emptyCaption
+                       [(AlignDefault,ColWidthDefault),(AlignDefault,ColWidthDefault)]
+                       (TableHead nullAttr [toRow [mempty, mempty]])
+                       [TableBody nullAttr 0 [] $ map toRow rows]
+                       (TableFoot nullAttr [])
               =?>
               unlines [ " Para 1.1 | Para 1.2"
                       , " Para 2.1 | Para 2.2"
@@ -389,12 +392,16 @@ tests = [ testGroup "block elements"
                       , " Para 2.1 |  Para 2.2"
                       ]
             , "table with header and caption" =:
-              let caption = "Table 1"
-                  headers = [plain "header 1", plain "header 2"]
-                  rows = [[para "Para 1.1", para  "Para 1.2"]
-                         ,[para "Para 2.1", para  "Para 2.2"]]
-              in table caption [(AlignDefault,0.0),(AlignDefault,0.0)]
-                        headers rows
+              let capt = simpleCaption $ plain "Table 1"
+                  toRow = Row nullAttr . map simpleCell
+                  headers = [toRow [plain "header 1", plain "header 2"]]
+                  rows = map toRow [[para "Para 1.1", para  "Para 1.2"]
+                                   ,[para "Para 2.1", para  "Para 2.2"]]
+              in table capt
+                       [(AlignDefault,ColWidthDefault),(AlignDefault,ColWidthDefault)]
+                       (TableHead nullAttr headers)
+                       [TableBody nullAttr 0 [] rows]
+                       (TableFoot nullAttr [])
               =?> unlines [ " header 1 || header 2"
                           , " Para 1.1 |  Para 1.2"
                           , " Para 2.1 |  Para 2.2"
