@@ -1,4 +1,4 @@
-function Doc (doc)
+function Pandoc (doc)
   local meta = {}
   local hello = { pandoc.Str "Hello,", pandoc.Space(), pandoc.Str "World!" }
   local blocks = { pandoc.Para(hello) }
